﻿using StudiApp.Models;
using StudiApp.Services.Converters;

namespace StudiApp.Test.Unit.ConverterTests
{
    [TestFixture]
    public class AverageMarkConverterTest
    {
        [TestCase(1.0, 6.0, 0.5, 0.5, 3.5)]
        [TestCase(1.0, 6.0, 0.3, 0.7, 4.5)]
        [TestCase(1.0, 6.0, 0.7, 0.3, 2.5)]
        [TestCase(5.0, 6.0, 0.35, 0.65, 5.65)]
        [TestCase(5.0, 6.0, 0.01, 0.99, 5.99)]
        [TestCase(4.0, 6.0, 1, 1, 5.0)]
        [TestCase(4.5, 5, 0.2, 0.4, 4.83)]
        public void AverageMarkTest(double mark1, double mark2, double percentage1, double percentage2, double expected)
        {
            var m1 = new Mark()
            {
                Value = mark1,
                Percentage = percentage1
            };

            var m2 = new Mark()
            {
                Value = mark2,
                Percentage = percentage2
            };

            var markList = new List<Mark>() { m1, m2 };

            Assert.That(AverageMarkConverter.MarksToAverage(markList), Is.EqualTo(expected));
        }

        [TestCase(0.9,1.0)]
        [TestCase(6.1,1.0)]
        public void AverageMarkValueExceptionTest(double mark, double percentage)
        {
            var m1 = new Mark()
            {
                Value = mark,
                Percentage = percentage
            };

            var markList = new List<Mark>() { m1};

            Assert.That(() => AverageMarkConverter.MarksToAverage(markList),
                Throws.TypeOf<ArgumentException>().With.Message.EqualTo("A Value in the marks list is over 6.0 or under 1.0"));
        }

        [TestCase(5, 0.0)]
        [TestCase(6, 1.1)]
        public void AverageMarkPercentageExceptionTest(double mark, double percentage)
        {
            var m1 = new Mark()
            {
                Value = mark,
                Percentage = percentage
            };

            var markList = new List<Mark>() { m1 };

            Assert.That(() => AverageMarkConverter.MarksToAverage(markList),
                Throws.TypeOf<ArgumentException>().With.Message.EqualTo("A Percentage in the marks list is over 100% (1.0) or under 1% (0.01)"));
        }
    }
}
