﻿using StudiApp.Models;
using StudiApp.Services.Repository;
using StudiApp.ViewModels;
using StudiApp.Interfaces;
using Microsoft.Extensions.Logging.Abstractions;
using Moq;

namespace StudiApp.Test.Unit.ViewModelsTests;

[TestFixture]
public class MarksOverviewViewModelTest
{
    private MarksOverviewViewModel vm;
    private Semester semester1;
    private Semester semester2;
    private List<Semester> semesters;
    private List<Course> semester1Courses;
    private List<Course> semester2Courses;
    private List<Mark> semester1Marks;
    private List<Mark> semester2Marks;

    [SetUp]
    public void Setup()
    {
        semester1 = new Semester()
        {
            DateRange = new DateRange() { Start = new DateTime(2032, 1, 1), End = new DateTime(2023, 6, 1) },
            Name = "Semester 1",
            Id = 1
        };
        semester2 = new Semester()
        {
            DateRange = new DateRange() { Start = new DateTime(2032, 6, 2), End = new DateTime(2023, 12, 1) },
            Name = "Semester 2",
            Id = 2
        };
        semesters = new List<Semester>() { semester1, semester2 };


        var mark1 = new Mark { Id = 1, Value = 4, Description = "Exam 1", Percentage = 0.5 };
        var mark2 = new Mark { Id = 2, Value = 4, Description = "Exam 2", Percentage = 0.5 };
        semester1Marks = new List<Mark>() { mark1 };
        semester2Marks = new List<Mark>() { mark2 };
        var allMarks = new List<Mark>() { mark1, mark2 };

        var course1 = new Course() { Id = 1, Name = "Course 1", Ects = 4, IsPassed = true, SemesterId = 1, Marks = semester1Marks };
        var course2 = new Course() { Id = 2, Name = "Course 2", Ects = 2, IsPassed = false, SemesterId = 2, Marks = semester2Marks };
        semester1Courses = new List<Course>() { course1 };
        semester2Courses = new List<Course>() { course2 };
        var allCourses = new List<Course>() { course1, course2 };


        var semesterRepository = new Mock<IRepository<Semester, int>>();
        semesterRepository.Setup(r => r.GetAll())
            .Returns(Task.FromResult(semesters));

        var courseRepository = new Mock<IRepository<Course, int>>();
        courseRepository.Setup(r => r.GetAll())
            .Returns(Task.FromResult(allCourses));

        var markRepository = new Mock<IRepository<Mark, int>>();
        markRepository.Setup(r => r.GetAll())
            .Returns(Task.FromResult(allMarks));

        vm = new MarksOverviewViewModel(
            new NullLogger<MarksOverviewViewModel>(),
            semesterRepository.Object,
            courseRepository.Object,
            markRepository.Object);
    }


    [Test]
    public void OnSelectedSemesterChangedTest()
    {
        vm.SelectedSemester = semester2;
        
        var actualId = vm.Marks[0].Id;
        var expectedId= semester2Marks[0].Id;
        Assert.That(actualId, Is.EqualTo(expectedId));
    }
}
